# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from ..types.event import Event
from .._base_client import AsyncPaginator, make_request_options
from ..types.event_list_params import EventListParams

__all__ = ["Events", "AsyncEvents"]


class Events(SyncAPIResource):
    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Event:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/events/{id}",
            options=options,
            cast_to=Event,
        )

    def list(
        self,
        query: EventListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[Event]:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/events",
            page=SyncPage[Event],
            options=options,
            model=Event,
        )


class AsyncEvents(AsyncAPIResource):
    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Event:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/events/{id}",
            options=options,
            cast_to=Event,
        )

    def list(
        self,
        query: EventListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[Event, AsyncPage[Event]]:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/events",
            page=AsyncPage[Event],
            options=options,
            model=Event,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.ledger_account import LedgerAccount
from ..types.ledger_account_list_params import LedgerAccountListParams
from ..types.ledger_account_create_params import LedgerAccountCreateParams
from ..types.ledger_account_update_params import LedgerAccountUpdateParams
from ..types.ledger_account_retrieve_params import LedgerAccountRetrieveParams

__all__ = ["LedgerAccounts", "AsyncLedgerAccounts"]


class LedgerAccounts(SyncAPIResource):
    def create(
        self,
        body: LedgerAccountCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccount:
        """Create a ledger account."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            "/api/ledger_accounts",
            body=body,
            options=options,
            cast_to=LedgerAccount,
        )

    def retrieve(
        self,
        id: str,
        query: LedgerAccountRetrieveParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> LedgerAccount:
        """Get details on a single ledger account."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/ledger_accounts/{id}",
            options=options,
            cast_to=LedgerAccount,
        )

    def update(
        self,
        id: str,
        body: LedgerAccountUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccount:
        """Update the details of a ledger account."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._patch(
            f"/api/ledger_accounts/{id}",
            body=body,
            options=options,
            cast_to=LedgerAccount,
        )

    def list(
        self,
        query: LedgerAccountListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[LedgerAccount]:
        """Get a list of ledger accounts."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/ledger_accounts",
            page=SyncPage[LedgerAccount],
            options=options,
            model=LedgerAccount,
        )

    def delete(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccount:
        """Delete a ledger account."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._delete(
            f"/api/ledger_accounts/{id}",
            options=options,
            cast_to=LedgerAccount,
        )


class AsyncLedgerAccounts(AsyncAPIResource):
    async def create(
        self,
        body: LedgerAccountCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccount:
        """Create a ledger account."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            "/api/ledger_accounts",
            body=body,
            options=options,
            cast_to=LedgerAccount,
        )

    async def retrieve(
        self,
        id: str,
        query: LedgerAccountRetrieveParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> LedgerAccount:
        """Get details on a single ledger account."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/ledger_accounts/{id}",
            options=options,
            cast_to=LedgerAccount,
        )

    async def update(
        self,
        id: str,
        body: LedgerAccountUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccount:
        """Update the details of a ledger account."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._patch(
            f"/api/ledger_accounts/{id}",
            body=body,
            options=options,
            cast_to=LedgerAccount,
        )

    def list(
        self,
        query: LedgerAccountListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[LedgerAccount, AsyncPage[LedgerAccount]]:
        """Get a list of ledger accounts."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/ledger_accounts",
            page=AsyncPage[LedgerAccount],
            options=options,
            model=LedgerAccount,
        )

    async def delete(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccount:
        """Delete a ledger account."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._delete(
            f"/api/ledger_accounts/{id}",
            options=options,
            cast_to=LedgerAccount,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.ledger_entry import LedgerEntry
from ..types.ledger_entry_list_params import LedgerEntryListParams

__all__ = ["LedgerEntries", "AsyncLedgerEntries"]


class LedgerEntries(SyncAPIResource):
    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerEntry:
        """Get details on a single ledger entry."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/ledger_entries/{id}",
            options=options,
            cast_to=LedgerEntry,
        )

    def list(
        self,
        query: LedgerEntryListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[LedgerEntry]:
        """Get a list of all ledger entries."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/ledger_entries",
            page=SyncPage[LedgerEntry],
            options=options,
            model=LedgerEntry,
        )


class AsyncLedgerEntries(AsyncAPIResource):
    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerEntry:
        """Get details on a single ledger entry."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/ledger_entries/{id}",
            options=options,
            cast_to=LedgerEntry,
        )

    def list(
        self,
        query: LedgerEntryListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[LedgerEntry, AsyncPage[LedgerEntry]]:
        """Get a list of all ledger entries."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/ledger_entries",
            page=AsyncPage[LedgerEntry],
            options=options,
            model=LedgerEntry,
        )



# File generated from our OpenAPI spec by Stainless.

from typing import Dict, List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "PartyAddress",
    "AccountDetails",
    "RoutingDetailsBankAddress",
    "RoutingDetails",
    "ContactDetails",
    "ExternalAccount",
]


class PartyAddress(BaseModel):
    country: Optional[str]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    created_at: str

    id: str

    line1: Optional[str]

    line2: Optional[str]

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    locality: Optional[str]
    """Locality or City."""

    object: str

    postal_code: Optional[str]
    """The postal code of the address."""

    region: Optional[str]
    """Region or State."""

    updated_at: str


class AccountDetails(BaseModel):
    account_number: str

    account_number_type: Literal["iban", "clabe", "wallet_address", "pan", "other"]
    """
    Supports iban and clabe, otherwise other if the bank account number is in a
    generic format.
    """

    created_at: str

    discarded_at: Optional[str]

    id: str

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    updated_at: str


class RoutingDetailsBankAddress(BaseModel):
    country: Optional[str]
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    created_at: str

    id: str

    line1: Optional[str]

    line2: Optional[str]

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    locality: Optional[str]
    """Locality or City."""

    object: str

    postal_code: Optional[str]
    """The postal code of the address."""

    region: Optional[str]
    """Region or State."""

    updated_at: str


class RoutingDetails(BaseModel):
    bank_address: Optional[RoutingDetailsBankAddress]

    bank_name: str

    created_at: str

    discarded_at: Optional[str]

    id: str

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    payment_type: Optional[
        Literal[
            "ach",
            "au_becs",
            "bacs",
            "book",
            "card",
            "check",
            "eft",
            "interac",
            "provxchange",
            "rtp",
            "sen",
            "sepa",
            "signet",
            "wire",
        ]
    ]
    """
    If the routing detail is to be used for a specific payment type this field will
    be populated, otherwise null.
    """

    routing_number: str
    """The routing number of the bank."""

    routing_number_type: Literal[
        "aba", "swift", "au_bsb", "ca_cpa", "cnaps", "gb_sort_code", "in_ifsc", "my_branch_code", "br_codigo"
    ]

    updated_at: str


class ContactDetails(BaseModel):
    contact_identifier: str

    contact_identifier_type: Literal["email", "phone_number"]

    created_at: str

    discarded_at: Optional[str]

    id: str

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    updated_at: str


class ExternalAccount(BaseModel):
    account_details: List[AccountDetails]

    account_type: Literal["checking", "other", "savings"]
    """Can be `checking`, `savings` or `other`."""

    contact_details: List[ContactDetails]

    counterparty_id: Optional[str]

    created_at: str

    discarded_at: Optional[str]

    id: str

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    name: Optional[str]
    """A nickname for the external account.

    This is only for internal usage and won't affect any payments
    """

    object: str

    party_address: Optional[PartyAddress]
    """The address associated with the owner or `null`."""

    party_name: str
    """The legal name of the entity which owns the account."""

    party_type: Optional[Literal["business", "individual"]]
    """Either `individual` or `business`."""

    routing_details: List[RoutingDetails]

    updated_at: str

    verification_status: Literal["pending_verification", "unverified", "verified"]



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NoneType, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.ledger_account_category import LedgerAccountCategory
from ..types.ledger_account_category_list_params import LedgerAccountCategoryListParams
from ..types.ledger_account_category_create_params import (
    LedgerAccountCategoryCreateParams,
)
from ..types.ledger_account_category_update_params import (
    LedgerAccountCategoryUpdateParams,
)

__all__ = ["LedgerAccountCategories", "AsyncLedgerAccountCategories"]


class LedgerAccountCategories(SyncAPIResource):
    def create(
        self,
        body: LedgerAccountCategoryCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccountCategory:
        """Create a ledger account category."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            "/api/ledger_account_categories",
            body=body,
            options=options,
            cast_to=LedgerAccountCategory,
        )

    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccountCategory:
        """Get the details on a single ledger account category."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/ledger_account_categories/{id}",
            options=options,
            cast_to=LedgerAccountCategory,
        )

    def update(
        self,
        id: str,
        body: LedgerAccountCategoryUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccountCategory:
        """Update the details of a ledger account category."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._patch(
            f"/api/ledger_account_categories/{id}",
            body=body,
            options=options,
            cast_to=LedgerAccountCategory,
        )

    def list(
        self,
        query: LedgerAccountCategoryListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[LedgerAccountCategory]:
        """Get a list of ledger account categories."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/ledger_account_categories",
            page=SyncPage[LedgerAccountCategory],
            options=options,
            model=LedgerAccountCategory,
        )

    def delete(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccountCategory:
        """Delete a ledger account category."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._delete(
            f"/api/ledger_account_categories/{id}",
            options=options,
            cast_to=LedgerAccountCategory,
        )

    def add_ledger_account(
        self,
        id: str,
        ledger_account_id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Add a ledger account category to an account."""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return self._put(
            f"/api/ledger_account_categories/{id}/ledger_accounts/{ledger_account_id}",
            options=options,
            cast_to=NoneType,
        )

    def add_nested_category(
        self,
        id: str,
        sub_category_id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Add a ledger account category to a ledger account category."""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return self._put(
            f"/api/ledger_account_categories/{id}/ledger_account_categories/{sub_category_id}",
            options=options,
            cast_to=NoneType,
        )

    def remove_ledger_account(
        self,
        id: str,
        ledger_account_id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Delete a ledger account category from an account."""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return self._delete(
            f"/api/ledger_account_categories/{id}/ledger_accounts/{ledger_account_id}",
            options=options,
            cast_to=NoneType,
        )

    def remove_nested_category(
        self,
        id: str,
        sub_category_id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Delete a ledger account category from a ledger account category."""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return self._delete(
            f"/api/ledger_account_categories/{id}/ledger_account_categories/{sub_category_id}",
            options=options,
            cast_to=NoneType,
        )


class AsyncLedgerAccountCategories(AsyncAPIResource):
    async def create(
        self,
        body: LedgerAccountCategoryCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccountCategory:
        """Create a ledger account category."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            "/api/ledger_account_categories",
            body=body,
            options=options,
            cast_to=LedgerAccountCategory,
        )

    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccountCategory:
        """Get the details on a single ledger account category."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/ledger_account_categories/{id}",
            options=options,
            cast_to=LedgerAccountCategory,
        )

    async def update(
        self,
        id: str,
        body: LedgerAccountCategoryUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccountCategory:
        """Update the details of a ledger account category."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._patch(
            f"/api/ledger_account_categories/{id}",
            body=body,
            options=options,
            cast_to=LedgerAccountCategory,
        )

    def list(
        self,
        query: LedgerAccountCategoryListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[LedgerAccountCategory, AsyncPage[LedgerAccountCategory]]:
        """Get a list of ledger account categories."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/ledger_account_categories",
            page=AsyncPage[LedgerAccountCategory],
            options=options,
            model=LedgerAccountCategory,
        )

    async def delete(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerAccountCategory:
        """Delete a ledger account category."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._delete(
            f"/api/ledger_account_categories/{id}",
            options=options,
            cast_to=LedgerAccountCategory,
        )

    async def add_ledger_account(
        self,
        id: str,
        ledger_account_id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Add a ledger account category to an account."""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._put(
            f"/api/ledger_account_categories/{id}/ledger_accounts/{ledger_account_id}",
            options=options,
            cast_to=NoneType,
        )

    async def add_nested_category(
        self,
        id: str,
        sub_category_id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Add a ledger account category to a ledger account category."""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._put(
            f"/api/ledger_account_categories/{id}/ledger_account_categories/{sub_category_id}",
            options=options,
            cast_to=NoneType,
        )

    async def remove_ledger_account(
        self,
        id: str,
        ledger_account_id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Delete a ledger account category from an account."""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._delete(
            f"/api/ledger_account_categories/{id}/ledger_accounts/{ledger_account_id}",
            options=options,
            cast_to=NoneType,
        )

    async def remove_nested_category(
        self,
        id: str,
        sub_category_id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Delete a ledger account category from a ledger account category."""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._delete(
            f"/api/ledger_account_categories/{id}/ledger_account_categories/{sub_category_id}",
            options=options,
            cast_to=NoneType,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.transaction import Transaction
from ..types.transaction_list_params import TransactionListParams
from ..types.transaction_update_params import TransactionUpdateParams

__all__ = ["Transactions", "AsyncTransactions"]


class Transactions(SyncAPIResource):
    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Transaction:
        """Get details on a single transaction."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/transactions/{id}",
            options=options,
            cast_to=Transaction,
        )

    def update(
        self,
        id: str,
        body: TransactionUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Transaction:
        """Update a single transaction."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._patch(
            f"/api/transactions/{id}",
            body=body,
            options=options,
            cast_to=Transaction,
        )

    def list(
        self,
        query: TransactionListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[Transaction]:
        """Get a list of all transactions."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/transactions",
            page=SyncPage[Transaction],
            options=options,
            model=Transaction,
        )


class AsyncTransactions(AsyncAPIResource):
    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Transaction:
        """Get details on a single transaction."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/transactions/{id}",
            options=options,
            cast_to=Transaction,
        )

    async def update(
        self,
        id: str,
        body: TransactionUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Transaction:
        """Update a single transaction."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._patch(
            f"/api/transactions/{id}",
            body=body,
            options=options,
            cast_to=Transaction,
        )

    def list(
        self,
        query: TransactionListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[Transaction, AsyncPage[Transaction]]:
        """Get a list of all transactions."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/transactions",
            page=AsyncPage[Transaction],
            options=options,
            model=Transaction,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union

from ..._types import NOT_GIVEN, Headers, Timeout, NotGiven
from ..._resource import SyncAPIResource, AsyncAPIResource
from ...pagination import SyncPage, AsyncPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.payment_orders.reversal import Reversal
from ...types.payment_orders.reversal_list_params import ReversalListParams

__all__ = ["Reversals", "AsyncReversals"]


class Reversals(SyncAPIResource):
    def list(
        self,
        payment_order_id: str,
        query: ReversalListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[Reversal]:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            f"/api/payment_orders/{payment_order_id}/reversals",
            page=SyncPage[Reversal],
            options=options,
            model=Reversal,
        )


class AsyncReversals(AsyncAPIResource):
    def list(
        self,
        payment_order_id: str,
        query: ReversalListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[Reversal, AsyncPage[Reversal]]:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            f"/api/payment_orders/{payment_order_id}/reversals",
            page=AsyncPage[Reversal],
            options=options,
            model=Reversal,
        )



# File generated from our OpenAPI spec by Stainless.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ReferenceNumbers", "ReturnObject"]


class ReferenceNumbers(BaseModel):
    created_at: str

    id: str

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    reference_number: str
    """The vendor reference number."""

    reference_number_type: Literal[
        "ach_original_trace_number",
        "ach_trace_number",
        "bankprov_payment_id",
        "bnk_dev_prenotification_id",
        "bnk_dev_transfer_id",
        "bofa_end_to_end_id",
        "bofa_transaction_id",
        "check_number",
        "cross_river_payment_id",
        "cross_river_transaction_id",
        "currencycloud_conversion_id",
        "currencycloud_payment_id",
        "dc_bank_transaction_id",
        "dwolla_transaction_id",
        "eft_trace_number",
        "fedwire_imad",
        "fedwire_omad",
        "first_republic_internal_id",
        "goldman_sachs_collection_request_id",
        "goldman_sachs_end_to_end_id",
        "goldman_sachs_payment_request_id",
        "goldman_sachs_request_id",
        "goldman_sachs_unique_payment_id",
        "interac_message_id",
        "jpmc_ccn",
        "jpmc_end_to_end_id",
        "jpmc_firm_root_id",
        "jpmc_p3_id",
        "jpmc_payment_batch_id",
        "jpmc_payment_information_id",
        "lob_check_id",
        "other",
        "partial_swift_mir",
        "pnc_clearing_reference",
        "pnc_instruction_id",
        "pnc_multipayment_id",
        "pnc_payment_trace_id",
        "rtp_instruction_id",
        "signet_api_reference_id",
        "signet_confirmation_id",
        "signet_request_id",
        "silvergate_payment_id",
        "swift_mir",
        "swift_uetr",
        "usbank_payment_id",
        "wells_fargo_payment_id",
        "wells_fargo_trace_number",
    ]
    """The type of the reference number. Referring to the vendor payment id."""

    updated_at: str


class ReturnObject(BaseModel):
    additional_information: Optional[str]
    """Some returns may include additional information from the bank.

    In these cases, this string will be present.
    """

    amount: int
    """Value in specified currency's smallest unit.

    e.g. $10 would be represented as 1000.
    """

    code: Optional[
        Literal[
            "901",
            "902",
            "903",
            "904",
            "905",
            "907",
            "908",
            "909",
            "910",
            "911",
            "912",
            "914",
            "R01",
            "R02",
            "R03",
            "R04",
            "R05",
            "R06",
            "R07",
            "R08",
            "R09",
            "R10",
            "R11",
            "R12",
            "R14",
            "R15",
            "R16",
            "R17",
            "R20",
            "R21",
            "R22",
            "R23",
            "R24",
            "R29",
            "R31",
            "R33",
            "R37",
            "R38",
            "R39",
            "R51",
            "R52",
            "R53",
            "currencycloud",
        ]
    ]
    """The return code. For ACH returns, this is the required ACH return code."""

    created_at: str

    currency: Literal[
        "AED",
        "AFN",
        "ALL",
        "AMD",
        "ANG",
        "AOA",
        "ARS",
        "AUD",
        "AWG",
        "AZN",
        "BAM",
        "BBD",
        "BCH",
        "BDT",
        "BGN",
        "BHD",
        "BIF",
        "BMD",
        "BND",
        "BOB",
        "BRL",
        "BSD",
        "BTC",
        "BTN",
        "BWP",
        "BYN",
        "BYR",
        "BZD",
        "CAD",
        "CDF",
        "CHF",
        "CLF",
        "CLP",
        "CNH",
        "CNY",
        "COP",
        "CRC",
        "CUC",
        "CUP",
        "CVE",
        "CZK",
        "DJF",
        "DKK",
        "DOP",
        "DZD",
        "EEK",
        "EGP",
        "ERN",
        "ETB",
        "EUR",
        "FJD",
        "FKP",
        "GBP",
        "GBX",
        "GEL",
        "GGP",
        "GHS",
        "GIP",
        "GMD",
        "GNF",
        "GTQ",
        "GYD",
        "HKD",
        "HNL",
        "HRK",
        "HTG",
        "HUF",
        "IDR",
        "ILS",
        "IMP",
        "INR",
        "IQD",
        "IRR",
        "ISK",
        "JEP",
        "JMD",
        "JOD",
        "JPY",
        "KES",
        "KGS",
        "KHR",
        "KMF",
        "KPW",
        "KRW",
        "KWD",
        "KYD",
        "KZT",
        "LAK",
        "LBP",
        "LKR",
        "LRD",
        "LSL",
        "LTL",
        "LVL",
        "LYD",
        "MAD",
        "MDL",
        "MGA",
        "MKD",
        "MMK",
        "MNT",
        "MOP",
        "MRO",
        "MRU",
        "MTL",
        "MUR",
        "MVR",
        "MWK",
        "MXN",
        "MYR",
        "MZN",
        "NAD",
        "NGN",
        "NIO",
        "NOK",
        "NPR",
        "NZD",
        "OMR",
        "PAB",
        "PEN",
        "PGK",
        "PHP",
        "PKR",
        "PLN",
        "PYG",
        "QAR",
        "RON",
        "RSD",
        "RUB",
        "RWF",
        "SAR",
        "SBD",
        "SCR",
        "SDG",
        "SEK",
        "SGD",
        "SHP",
        "SKK",
        "SLL",
        "SOS",
        "SRD",
        "SSP",
        "STD",
        "SVC",
        "SYP",
        "SZL",
        "THB",
        "TJS",
        "TMM",
        "TMT",
        "TND",
        "TOP",
        "TRY",
        "TTD",
        "TWD",
        "TZS",
        "UAH",
        "UGX",
        "USD",
        "UYU",
        "UZS",
        "VEF",
        "VES",
        "VND",
        "VUV",
        "WST",
        "XAF",
        "XAG",
        "XAU",
        "XBA",
        "XBB",
        "XBC",
        "XBD",
        "XCD",
        "XDR",
        "XFU",
        "XOF",
        "XPD",
        "XPF",
        "XPT",
        "XTS",
        "YER",
        "ZAR",
        "ZMK",
        "ZMW",
        "ZWD",
        "ZWL",
        "ZWN",
        "ZWR",
    ]
    """Currency that this transaction is denominated in."""

    date_of_death: Optional[str]
    """
    If the return code is `R14` or `R15` this is the date the deceased counterparty
    passed away.
    """

    id: str

    internal_account_id: Optional[str]
    """The ID of the relevant Internal Account."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    reason: Optional[str]
    """
    Often the bank will provide an explanation for the return, which is a short
    human readable string.
    """

    reference_numbers: List[ReferenceNumbers]
    """An array of Payment Reference objects."""

    returnable_id: Optional[str]
    """The ID of the object being returned or `null`."""

    returnable_type: Optional[Literal["incoming_payment_detail", "paper_item", "payment_order", "reversal"]]
    """The type of object being returned or `null`."""

    role: Literal["originating", "receiving"]
    """The role of the return, can be `originating` or `receiving`."""

    status: Literal["completed", "failed", "pending", "processing", "sent"]
    """The current status of the return."""

    transaction_id: Optional[str]
    """The ID of the relevant Transaction or `null`."""

    transaction_line_item_id: Optional[str]
    """The ID of the relevant Transaction Line Item or `null`."""

    type: Literal["ach", "ach_noc", "au_becs", "bacs", "eft", "interac", "manual", "paper_item", "wire"]
    """The type of return. Can be one of: `ach`, `paper_item`, `eft`, `wire`."""

    updated_at: str



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.expected_payment import ExpectedPayment
from ..types.expected_payment_list_params import ExpectedPaymentListParams
from ..types.expected_payment_create_params import ExpectedPaymentCreateParams
from ..types.expected_payment_update_params import ExpectedPaymentUpdateParams

__all__ = ["ExpectedPayments", "AsyncExpectedPayments"]


class ExpectedPayments(SyncAPIResource):
    def create(
        self,
        body: ExpectedPaymentCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExpectedPayment:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            "/api/expected_payments",
            body=body,
            options=options,
            cast_to=ExpectedPayment,
        )

    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExpectedPayment:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/expected_payments/{id}",
            options=options,
            cast_to=ExpectedPayment,
        )

    def update(
        self,
        id: str,
        body: ExpectedPaymentUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExpectedPayment:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._patch(
            f"/api/expected_payments/{id}",
            body=body,
            options=options,
            cast_to=ExpectedPayment,
        )

    def list(
        self,
        query: ExpectedPaymentListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[ExpectedPayment]:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/expected_payments",
            page=SyncPage[ExpectedPayment],
            options=options,
            model=ExpectedPayment,
        )

    def delete(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExpectedPayment:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._delete(
            f"/api/expected_payments/{id}",
            options=options,
            cast_to=ExpectedPayment,
        )


class AsyncExpectedPayments(AsyncAPIResource):
    async def create(
        self,
        body: ExpectedPaymentCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExpectedPayment:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            "/api/expected_payments",
            body=body,
            options=options,
            cast_to=ExpectedPayment,
        )

    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExpectedPayment:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/expected_payments/{id}",
            options=options,
            cast_to=ExpectedPayment,
        )

    async def update(
        self,
        id: str,
        body: ExpectedPaymentUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExpectedPayment:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._patch(
            f"/api/expected_payments/{id}",
            body=body,
            options=options,
            cast_to=ExpectedPayment,
        )

    def list(
        self,
        query: ExpectedPaymentListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[ExpectedPayment, AsyncPage[ExpectedPayment]]:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/expected_payments",
            page=AsyncPage[ExpectedPayment],
            options=options,
            model=ExpectedPayment,
        )

    async def delete(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExpectedPayment:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._delete(
            f"/api/expected_payments/{id}",
            options=options,
            cast_to=ExpectedPayment,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.ledger_transaction import LedgerTransaction
from ..types.ledger_transaction_list_params import LedgerTransactionListParams
from ..types.ledger_transaction_create_params import LedgerTransactionCreateParams
from ..types.ledger_transaction_update_params import LedgerTransactionUpdateParams

__all__ = ["LedgerTransactions", "AsyncLedgerTransactions"]


class LedgerTransactions(SyncAPIResource):
    def create(
        self,
        body: LedgerTransactionCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerTransaction:
        """Create a ledger transaction."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            "/api/ledger_transactions",
            body=body,
            options=options,
            cast_to=LedgerTransaction,
        )

    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerTransaction:
        """Get details on a single ledger transaction."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/ledger_transactions/{id}",
            options=options,
            cast_to=LedgerTransaction,
        )

    def update(
        self,
        id: str,
        body: LedgerTransactionUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerTransaction:
        """Update the details of a ledger transaction."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._patch(
            f"/api/ledger_transactions/{id}",
            body=body,
            options=options,
            cast_to=LedgerTransaction,
        )

    def list(
        self,
        query: LedgerTransactionListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[LedgerTransaction]:
        """Get a list of ledger transactions."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/ledger_transactions",
            page=SyncPage[LedgerTransaction],
            options=options,
            model=LedgerTransaction,
        )


class AsyncLedgerTransactions(AsyncAPIResource):
    async def create(
        self,
        body: LedgerTransactionCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerTransaction:
        """Create a ledger transaction."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            "/api/ledger_transactions",
            body=body,
            options=options,
            cast_to=LedgerTransaction,
        )

    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerTransaction:
        """Get details on a single ledger transaction."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/ledger_transactions/{id}",
            options=options,
            cast_to=LedgerTransaction,
        )

    async def update(
        self,
        id: str,
        body: LedgerTransactionUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LedgerTransaction:
        """Update the details of a ledger transaction."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._patch(
            f"/api/ledger_transactions/{id}",
            body=body,
            options=options,
            cast_to=LedgerTransaction,
        )

    def list(
        self,
        query: LedgerTransactionListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[LedgerTransaction, AsyncPage[LedgerTransaction]]:
        """Get a list of ledger transactions."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/ledger_transactions",
            page=AsyncPage[LedgerTransaction],
            options=options,
            model=LedgerTransaction,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.incoming_payment_detail import IncomingPaymentDetail
from ..types.incoming_payment_detail_list_params import IncomingPaymentDetailListParams
from ..types.incoming_payment_detail_update_params import (
    IncomingPaymentDetailUpdateParams,
)

__all__ = ["IncomingPaymentDetails", "AsyncIncomingPaymentDetails"]


class IncomingPaymentDetails(SyncAPIResource):
    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> IncomingPaymentDetail:
        """Get an existing Incoming Payment Detail."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/incoming_payment_details/{id}",
            options=options,
            cast_to=IncomingPaymentDetail,
        )

    def update(
        self,
        id: str,
        body: IncomingPaymentDetailUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> IncomingPaymentDetail:
        """Update an existing Incoming Payment Detail."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._patch(
            f"/api/incoming_payment_details/{id}",
            body=body,
            options=options,
            cast_to=IncomingPaymentDetail,
        )

    def list(
        self,
        query: IncomingPaymentDetailListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[IncomingPaymentDetail]:
        """Get a list of Incoming Payment Details."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/incoming_payment_details",
            page=SyncPage[IncomingPaymentDetail],
            options=options,
            model=IncomingPaymentDetail,
        )


class AsyncIncomingPaymentDetails(AsyncAPIResource):
    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> IncomingPaymentDetail:
        """Get an existing Incoming Payment Detail."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/incoming_payment_details/{id}",
            options=options,
            cast_to=IncomingPaymentDetail,
        )

    async def update(
        self,
        id: str,
        body: IncomingPaymentDetailUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> IncomingPaymentDetail:
        """Update an existing Incoming Payment Detail."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._patch(
            f"/api/incoming_payment_details/{id}",
            body=body,
            options=options,
            cast_to=IncomingPaymentDetail,
        )

    def list(
        self,
        query: IncomingPaymentDetailListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[IncomingPaymentDetail, AsyncPage[IncomingPaymentDetail]]:
        """Get a list of Incoming Payment Details."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/incoming_payment_details",
            page=AsyncPage[IncomingPaymentDetail],
            options=options,
            model=IncomingPaymentDetail,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union

from .._types import NOT_GIVEN, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from .._base_client import make_request_options
from ..types.routing_number_lookup_request import RoutingNumberLookupRequest
from ..types.validation_validate_routing_number_params import (
    ValidationValidateRoutingNumberParams,
)

__all__ = ["Validations", "AsyncValidations"]


class Validations(SyncAPIResource):
    def validate_routing_number(
        self,
        query: ValidationValidateRoutingNumberParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> RoutingNumberLookupRequest:
        """
        Validates the routing number information supplied without creating a routing
        detail
        """
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            "/api/validations/routing_numbers",
            options=options,
            cast_to=RoutingNumberLookupRequest,
        )


class AsyncValidations(AsyncAPIResource):
    async def validate_routing_number(
        self,
        query: ValidationValidateRoutingNumberParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> RoutingNumberLookupRequest:
        """
        Validates the routing number information supplied without creating a routing
        detail
        """
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            "/api/validations/routing_numbers",
            options=options,
            cast_to=RoutingNumberLookupRequest,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NoneType, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.counterparty import Counterparty
from ..types.counterparty_list_params import CounterpartyListParams
from ..types.counterparty_create_params import CounterpartyCreateParams
from ..types.counterparty_update_params import CounterpartyUpdateParams
from ..types.counterparty_collect_account_params import CounterpartyCollectAccountParams
from ..types.counterparty_collect_account_response import (
    CounterpartyCollectAccountResponse,
)

__all__ = ["Counterparties", "AsyncCounterparties"]


class Counterparties(SyncAPIResource):
    def create(
        self,
        body: CounterpartyCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Counterparty:
        """Create a new counterparty."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            "/api/counterparties",
            body=body,
            options=options,
            cast_to=Counterparty,
        )

    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Counterparty:
        """Get details on a single counterparty."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/counterparties/{id}",
            options=options,
            cast_to=Counterparty,
        )

    def update(
        self,
        id: str,
        body: CounterpartyUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Counterparty:
        """Updates a given counterparty with new information."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._patch(
            f"/api/counterparties/{id}",
            body=body,
            options=options,
            cast_to=Counterparty,
        )

    def list(
        self,
        query: CounterpartyListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[Counterparty]:
        """Get a paginated list of all counterparties."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/counterparties",
            page=SyncPage[Counterparty],
            options=options,
            model=Counterparty,
        )

    def delete(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Deletes a given counterparty."""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return self._delete(
            f"/api/counterparties/{id}",
            options=options,
            cast_to=NoneType,
        )

    def collect_account(
        self,
        id: str,
        body: CounterpartyCollectAccountParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> CounterpartyCollectAccountResponse:
        """Send an email requesting account details."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            f"/api/counterparties/{id}/collect_account",
            body=body,
            options=options,
            cast_to=CounterpartyCollectAccountResponse,
        )


class AsyncCounterparties(AsyncAPIResource):
    async def create(
        self,
        body: CounterpartyCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Counterparty:
        """Create a new counterparty."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            "/api/counterparties",
            body=body,
            options=options,
            cast_to=Counterparty,
        )

    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Counterparty:
        """Get details on a single counterparty."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/counterparties/{id}",
            options=options,
            cast_to=Counterparty,
        )

    async def update(
        self,
        id: str,
        body: CounterpartyUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> Counterparty:
        """Updates a given counterparty with new information."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._patch(
            f"/api/counterparties/{id}",
            body=body,
            options=options,
            cast_to=Counterparty,
        )

    def list(
        self,
        query: CounterpartyListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[Counterparty, AsyncPage[Counterparty]]:
        """Get a paginated list of all counterparties."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/counterparties",
            page=AsyncPage[Counterparty],
            options=options,
            model=Counterparty,
        )

    async def delete(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Deletes a given counterparty."""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._delete(
            f"/api/counterparties/{id}",
            options=options,
            cast_to=NoneType,
        )

    async def collect_account(
        self,
        id: str,
        body: CounterpartyCollectAccountParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> CounterpartyCollectAccountResponse:
        """Send an email requesting account details."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            f"/api/counterparties/{id}/collect_account",
            body=body,
            options=options,
            cast_to=CounterpartyCollectAccountResponse,
        )



# File generated from our OpenAPI spec by Stainless.

from typing import List, Type, Generic, Mapping, TypeVar, Optional
from typing_extensions import TypedDict

from httpx import Response
from pydantic import BaseModel

from ._types import ModelT
from ._base_client import BasePage, BaseSyncPage, BaseAsyncPage

__all__ = ["PageParams", "SyncPage", "AsyncPage"]

_BaseModelT = TypeVar("_BaseModelT", bound=BaseModel)


class PageParams(TypedDict, total=False):
    after_cursor: Optional[str]

    per_page: int


class SyncPage(BaseSyncPage[ModelT], BasePage[ModelT, PageParams], Generic[ModelT]):
    items: List[ModelT]
    per_page: Optional[int]
    after_cursor: Optional[str]

    def _get_page_items(self) -> List[ModelT]:
        return self.items

    def next_page_params(self) -> Optional[PageParams]:
        cursor = self.after_cursor
        if not cursor:
            return None

        return {"after_cursor": cursor}

    @classmethod
    def build(cls: Type[_BaseModelT], *, response: Response, data: object) -> _BaseModelT:
        return cls.parse_obj(
            {
                **(data if isinstance(data, Mapping) else {"items": data}),
                "per_page": response.headers.get("X-Per-Page"),
                "after_cursor": response.headers.get("X-After-Cursor"),
            }
        )


class AsyncPage(BaseAsyncPage[ModelT], BasePage[ModelT, PageParams], Generic[ModelT]):
    items: List[ModelT]
    per_page: Optional[int]
    after_cursor: Optional[str]

    def _get_page_items(self) -> List[ModelT]:
        return self.items

    def next_page_params(self) -> Optional[PageParams]:
        cursor = self.after_cursor
        if not cursor:
            return None

        return {"after_cursor": cursor}

    @classmethod
    def build(cls: Type[_BaseModelT], *, response: Response, data: object) -> _BaseModelT:
        return cls.parse_obj(
            {
                **(data if isinstance(data, Mapping) else {"items": data}),
                "per_page": response.headers.get("X-Per-Page"),
                "after_cursor": response.headers.get("X-After-Cursor"),
            }
        )



# File generated from our OpenAPI spec by Stainless.

import os
import base64
from typing import Dict, Union, Optional

from . import resources
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Query,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._version import __version__
from ._base_client import (
    DEFAULT_TIMEOUT,
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
    make_request_options,
)
from .types.ping_response import PingResponse

__all__ = [
    "ModernTreasury",
    "AsyncModernTreasury",
    "Client",
    "AsyncClient",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "resources",
]


class ModernTreasury(SyncAPIClient):
    counterparties: resources.Counterparties
    events: resources.Events
    expected_payments: resources.ExpectedPayments
    external_accounts: resources.ExternalAccounts
    incoming_payment_details: resources.IncomingPaymentDetails
    documents: resources.Documents
    internal_accounts: resources.InternalAccounts
    ledgers: resources.Ledgers
    ledger_account_categories: resources.LedgerAccountCategories
    ledger_accounts: resources.LedgerAccounts
    ledger_entries: resources.LedgerEntries
    ledger_transactions: resources.LedgerTransactions
    line_items: resources.LineItems
    payment_orders: resources.PaymentOrders
    returns: resources.Returns
    transactions: resources.Transactions
    validations: resources.Validations
    paper_items: resources.PaperItems
    webhooks: resources.Webhooks

    def __init__(
        self,
        *,
        organization_id: Optional[str] = None,
        webhook_key: Optional[str] = None,
        base_url: Optional[str] = None,
        api_key: Optional[str] = None,
        timeout: Union[float, Timeout, None] = DEFAULT_TIMEOUT,
        max_retries: int = DEFAULT_MAX_RETRIES,
        # See httpx documentation for [custom transports](https://www.python-httpx.org/advanced/#custom-transports)
        transport: Optional[Transport] = None,
        # See httpx documentation for [proxies](https://www.python-httpx.org/advanced/#http-proxying)
        proxies: Optional[ProxiesTypes] = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Modern Treasury client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `MODERN_TREASURY_API_KEY`
        - `organization_id` from `MODERN_TREASURY_ORGANIZATION_ID`
        - `webhook_key` from `MODERN_TREASURY_WEBHOOK_KEY`
        """
        api_key = api_key or os.environ.get("MODERN_TREASURY_API_KEY", "")
        if not api_key:
            raise Exception("No API key provided")

        if base_url is None:
            base_url = "https://app.moderntreasury.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            api_key=api_key,
            max_retries=max_retries,
            timeout=timeout,
            transport=transport,
            proxies=proxies,
            _strict_response_validation=_strict_response_validation,
        )

        self.organization_id = organization_id or os.environ.get("MODERN_TREASURY_ORGANIZATION_ID")
        if self.organization_id is None:
            raise ValueError(
                "The organization_id client option must be set either by passing organization_id to the client or by setting the MODERN_TREASURY_ORGANIZATION_ID environment variable"
            )

        self.webhook_key = webhook_key or os.environ.get("MODERN_TREASURY_WEBHOOK_KEY")

        self._idempotency_header = "Idempotency-Key"

        self.counterparties = resources.Counterparties(self)
        self.events = resources.Events(self)
        self.expected_payments = resources.ExpectedPayments(self)
        self.external_accounts = resources.ExternalAccounts(self)
        self.incoming_payment_details = resources.IncomingPaymentDetails(self)
        self.documents = resources.Documents(self)
        self.internal_accounts = resources.InternalAccounts(self)
        self.ledgers = resources.Ledgers(self)
        self.ledger_account_categories = resources.LedgerAccountCategories(self)
        self.ledger_accounts = resources.LedgerAccounts(self)
        self.ledger_entries = resources.LedgerEntries(self)
        self.ledger_transactions = resources.LedgerTransactions(self)
        self.line_items = resources.LineItems(self)
        self.payment_orders = resources.PaymentOrders(self)
        self.returns = resources.Returns(self)
        self.transactions = resources.Transactions(self)
        self.validations = resources.Validations(self)
        self.paper_items = resources.PaperItems(self)
        self.webhooks = resources.Webhooks(self)

    @property
    def qs(self) -> Querystring:
        return Querystring(array_format="repeat")

    @property
    def auth_headers(self) -> Dict[str, str]:
        key = f"{self.organization_id}:{self.api_key}".encode("ascii")
        header = f"Basic {base64.b64encode(key).decode('ascii')}"
        return {"Authorization": header}

    def ping(
        self,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> PingResponse:
        """
        A test endpoint often used to confirm credentials and headers are being passed
        in correctly.
        """
        options = make_request_options(headers, max_retries, timeout, query)
        return self.get(
            "/api/ping",
            options=options,
            cast_to=PingResponse,
        )


class AsyncModernTreasury(AsyncAPIClient):
    counterparties: resources.AsyncCounterparties
    events: resources.AsyncEvents
    expected_payments: resources.AsyncExpectedPayments
    external_accounts: resources.AsyncExternalAccounts
    incoming_payment_details: resources.AsyncIncomingPaymentDetails
    documents: resources.AsyncDocuments
    internal_accounts: resources.AsyncInternalAccounts
    ledgers: resources.AsyncLedgers
    ledger_account_categories: resources.AsyncLedgerAccountCategories
    ledger_accounts: resources.AsyncLedgerAccounts
    ledger_entries: resources.AsyncLedgerEntries
    ledger_transactions: resources.AsyncLedgerTransactions
    line_items: resources.AsyncLineItems
    payment_orders: resources.AsyncPaymentOrders
    returns: resources.AsyncReturns
    transactions: resources.AsyncTransactions
    validations: resources.AsyncValidations
    paper_items: resources.AsyncPaperItems
    webhooks: resources.AsyncWebhooks

    def __init__(
        self,
        *,
        organization_id: Optional[str] = None,
        webhook_key: Optional[str] = None,
        base_url: Optional[str] = None,
        api_key: Optional[str] = None,
        timeout: Union[float, Timeout, None] = DEFAULT_TIMEOUT,
        max_retries: int = DEFAULT_MAX_RETRIES,
        # See httpx documentation for [custom transports](https://www.python-httpx.org/advanced/#custom-transports)
        transport: Optional[Transport] = None,
        # See httpx documentation for [proxies](https://www.python-httpx.org/advanced/#http-proxying)
        proxies: Optional[ProxiesTypes] = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async Modern Treasury client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `MODERN_TREASURY_API_KEY`
        - `organization_id` from `MODERN_TREASURY_ORGANIZATION_ID`
        - `webhook_key` from `MODERN_TREASURY_WEBHOOK_KEY`
        """
        api_key = api_key or os.environ.get("MODERN_TREASURY_API_KEY", "")
        if not api_key:
            raise Exception("No API key provided")

        if base_url is None:
            base_url = "https://app.moderntreasury.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            api_key=api_key,
            max_retries=max_retries,
            timeout=timeout,
            transport=transport,
            proxies=proxies,
            _strict_response_validation=_strict_response_validation,
        )

        self.organization_id = organization_id or os.environ.get("MODERN_TREASURY_ORGANIZATION_ID")
        if self.organization_id is None:
            raise ValueError(
                "The organization_id client option must be set either by passing organization_id to the client or by setting the MODERN_TREASURY_ORGANIZATION_ID environment variable"
            )

        self.webhook_key = webhook_key or os.environ.get("MODERN_TREASURY_WEBHOOK_KEY")

        self._idempotency_header = "Idempotency-Key"

        self.counterparties = resources.AsyncCounterparties(self)
        self.events = resources.AsyncEvents(self)
        self.expected_payments = resources.AsyncExpectedPayments(self)
        self.external_accounts = resources.AsyncExternalAccounts(self)
        self.incoming_payment_details = resources.AsyncIncomingPaymentDetails(self)
        self.documents = resources.AsyncDocuments(self)
        self.internal_accounts = resources.AsyncInternalAccounts(self)
        self.ledgers = resources.AsyncLedgers(self)
        self.ledger_account_categories = resources.AsyncLedgerAccountCategories(self)
        self.ledger_accounts = resources.AsyncLedgerAccounts(self)
        self.ledger_entries = resources.AsyncLedgerEntries(self)
        self.ledger_transactions = resources.AsyncLedgerTransactions(self)
        self.line_items = resources.AsyncLineItems(self)
        self.payment_orders = resources.AsyncPaymentOrders(self)
        self.returns = resources.AsyncReturns(self)
        self.transactions = resources.AsyncTransactions(self)
        self.validations = resources.AsyncValidations(self)
        self.paper_items = resources.AsyncPaperItems(self)
        self.webhooks = resources.AsyncWebhooks(self)

    @property
    def qs(self) -> Querystring:
        return Querystring(array_format="repeat")

    @property
    def auth_headers(self) -> Dict[str, str]:
        key = f"{self.organization_id}:{self.api_key}".encode("ascii")
        header = f"Basic {base64.b64encode(key).decode('ascii')}"
        return {"Authorization": header}

    async def ping(
        self,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> PingResponse:
        """
        A test endpoint often used to confirm credentials and headers are being passed
        in correctly.
        """
        options = make_request_options(headers, max_retries, timeout, query)
        return await self.get(
            "/api/ping",
            options=options,
            cast_to=PingResponse,
        )


Client = ModernTreasury

AsyncClient = AsyncModernTreasury



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.return_object import ReturnObject
from ..types.return_list_params import ReturnListParams
from ..types.return_create_params import ReturnCreateParams

__all__ = ["Returns", "AsyncReturns"]


class Returns(SyncAPIResource):
    def create(
        self,
        body: ReturnCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ReturnObject:
        """Create a return."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            "/api/returns",
            body=body,
            options=options,
            cast_to=ReturnObject,
        )

    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ReturnObject:
        """Get a single return."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/returns/{id}",
            options=options,
            cast_to=ReturnObject,
        )

    def list(
        self,
        query: ReturnListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[ReturnObject]:
        """Get a list of returns."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/returns",
            page=SyncPage[ReturnObject],
            options=options,
            model=ReturnObject,
        )


class AsyncReturns(AsyncAPIResource):
    async def create(
        self,
        body: ReturnCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ReturnObject:
        """Create a return."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            "/api/returns",
            body=body,
            options=options,
            cast_to=ReturnObject,
        )

    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ReturnObject:
        """Get a single return."""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/returns/{id}",
            options=options,
            cast_to=ReturnObject,
        )

    def list(
        self,
        query: ReturnListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[ReturnObject, AsyncPage[ReturnObject]]:
        """Get a list of returns."""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/returns",
            page=AsyncPage[ReturnObject],
            options=options,
            model=ReturnObject,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional

from .._types import NOT_GIVEN, Query, Headers, Timeout, NoneType, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.external_account import ExternalAccount
from ..types.external_account_list_params import ExternalAccountListParams
from ..types.external_account_create_params import ExternalAccountCreateParams
from ..types.external_account_update_params import ExternalAccountUpdateParams
from ..types.external_account_verify_params import ExternalAccountVerifyParams
from ..types.external_account_complete_verification_params import (
    ExternalAccountCompleteVerificationParams,
)

__all__ = ["ExternalAccounts", "AsyncExternalAccounts"]


class ExternalAccounts(SyncAPIResource):
    def create(
        self,
        body: ExternalAccountCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExternalAccount:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            "/api/external_accounts",
            body=body,
            options=options,
            cast_to=ExternalAccount,
        )

    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExternalAccount:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/external_accounts/{id}",
            options=options,
            cast_to=ExternalAccount,
        )

    def update(
        self,
        id: str,
        body: ExternalAccountUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExternalAccount:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._patch(
            f"/api/external_accounts/{id}",
            body=body,
            options=options,
            cast_to=ExternalAccount,
        )

    def list(
        self,
        query: ExternalAccountListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[ExternalAccount]:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/external_accounts",
            page=SyncPage[ExternalAccount],
            options=options,
            model=ExternalAccount,
        )

    def delete(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return self._delete(
            f"/api/external_accounts/{id}",
            options=options,
            cast_to=NoneType,
        )

    def complete_verification(
        self,
        id: str,
        body: ExternalAccountCompleteVerificationParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExternalAccount:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            f"/api/external_accounts/{id}/complete_verification",
            body=body,
            options=options,
            cast_to=ExternalAccount,
        )

    def verify(
        self,
        id: str,
        body: ExternalAccountVerifyParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExternalAccount:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            f"/api/external_accounts/{id}/verify",
            body=body,
            options=options,
            cast_to=ExternalAccount,
        )


class AsyncExternalAccounts(AsyncAPIResource):
    async def create(
        self,
        body: ExternalAccountCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExternalAccount:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            "/api/external_accounts",
            body=body,
            options=options,
            cast_to=ExternalAccount,
        )

    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExternalAccount:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/external_accounts/{id}",
            options=options,
            cast_to=ExternalAccount,
        )

    async def update(
        self,
        id: str,
        body: ExternalAccountUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExternalAccount:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._patch(
            f"/api/external_accounts/{id}",
            body=body,
            options=options,
            cast_to=ExternalAccount,
        )

    def list(
        self,
        query: ExternalAccountListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[ExternalAccount, AsyncPage[ExternalAccount]]:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/external_accounts",
            page=AsyncPage[ExternalAccount],
            options=options,
            model=ExternalAccount,
        )

    async def delete(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._delete(
            f"/api/external_accounts/{id}",
            options=options,
            cast_to=NoneType,
        )

    async def complete_verification(
        self,
        id: str,
        body: ExternalAccountCompleteVerificationParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExternalAccount:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            f"/api/external_accounts/{id}/complete_verification",
            body=body,
            options=options,
            cast_to=ExternalAccount,
        )

    async def verify(
        self,
        id: str,
        body: ExternalAccountVerifyParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> ExternalAccount:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            f"/api/external_accounts/{id}/verify",
            body=body,
            options=options,
            cast_to=ExternalAccount,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal

from .._types import NOT_GIVEN, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.document import Document
from ..types.document_list_params import DocumentListParams

__all__ = ["Documents", "AsyncDocuments"]


class Documents(SyncAPIResource):
    def list(
        self,
        documentable_type: Literal[
            "cases",
            "counterparties",
            "expected_payments",
            "external_accounts",
            "internal_accounts",
            "organizations",
            "paper_items",
            "payment_orders",
            "transactions",
        ],
        documentable_id: str,
        query: DocumentListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[Document]:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            f"/api/{documentable_type}/{documentable_id}/documents",
            page=SyncPage[Document],
            options=options,
            model=Document,
        )


class AsyncDocuments(AsyncAPIResource):
    def list(
        self,
        documentable_type: Literal[
            "cases",
            "counterparties",
            "expected_payments",
            "external_accounts",
            "internal_accounts",
            "organizations",
            "paper_items",
            "payment_orders",
            "transactions",
        ],
        documentable_id: str,
        query: DocumentListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[Document, AsyncPage[Document]]:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            f"/api/{documentable_type}/{documentable_id}/documents",
            page=AsyncPage[Document],
            options=options,
            model=Document,
        )



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal

from .._types import NOT_GIVEN, Query, Headers, Timeout, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.line_item import LineItem
from ..types.line_item_list_params import LineItemListParams
from ..types.line_item_update_params import LineItemUpdateParams

__all__ = ["LineItems", "AsyncLineItems"]


class LineItems(SyncAPIResource):
    def retrieve(
        self,
        itemizable_type: Literal["expected_payments", "payment_orders"],
        itemizable_id: str,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LineItem:
        """Get a single line item"""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/{itemizable_type}/{itemizable_id}/line_items/{id}",
            options=options,
            cast_to=LineItem,
        )

    def update(
        self,
        itemizable_type: Literal["expected_payments", "payment_orders"],
        itemizable_id: str,
        id: str,
        body: LineItemUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LineItem:
        options = make_request_options(headers, max_retries, timeout, query)
        return self._patch(
            f"/api/{itemizable_type}/{itemizable_id}/line_items/{id}",
            body=body,
            options=options,
            cast_to=LineItem,
        )

    def list(
        self,
        itemizable_type: Literal["expected_payments", "payment_orders"],
        itemizable_id: str,
        query: LineItemListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[LineItem]:
        """Get a list of line items"""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            f"/api/{itemizable_type}/{itemizable_id}/line_items",
            page=SyncPage[LineItem],
            options=options,
            model=LineItem,
        )


class AsyncLineItems(AsyncAPIResource):
    async def retrieve(
        self,
        itemizable_type: Literal["expected_payments", "payment_orders"],
        itemizable_id: str,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LineItem:
        """Get a single line item"""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/{itemizable_type}/{itemizable_id}/line_items/{id}",
            options=options,
            cast_to=LineItem,
        )

    async def update(
        self,
        itemizable_type: Literal["expected_payments", "payment_orders"],
        itemizable_id: str,
        id: str,
        body: LineItemUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> LineItem:
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._patch(
            f"/api/{itemizable_type}/{itemizable_id}/line_items/{id}",
            body=body,
            options=options,
            cast_to=LineItem,
        )

    def list(
        self,
        itemizable_type: Literal["expected_payments", "payment_orders"],
        itemizable_id: str,
        query: LineItemListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[LineItem, AsyncPage[LineItem]]:
        """Get a list of line items"""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            f"/api/{itemizable_type}/{itemizable_id}/line_items",
            page=AsyncPage[LineItem],
            options=options,
            model=LineItem,
        )



from __future__ import annotations

from typing import Mapping, TypeVar, Iterable, Sequence, cast
from typing_extensions import TypeGuard

from ._types import FileTypes

_T = TypeVar("_T")


def flatten(t: Iterable[Iterable[_T]]) -> list[_T]:
    return [item for sublist in t for item in sublist]


def extract_files(
    # TODO: this needs to take Dict but variance issues.....
    # create protocol type ?
    query: Mapping[str, object],
    *,
    paths: Sequence[Sequence[str]],
) -> list[tuple[str, FileTypes]]:
    files: list[tuple[str, FileTypes]] = []
    for path in paths:
        files.extend(_extract_items(query, path, index=0, flattened_key=None))
    return files


def _extract_items(
    obj: object,
    path: Sequence[str],
    *,
    index: int,
    # TODO: rename
    flattened_key: str | None,
) -> list[tuple[str, FileTypes]]:
    try:
        key = path[index]
    except IndexError:
        # We have exhausted the path, return the entry we found.
        if not isinstance(obj, bytes) and not isinstance(obj, tuple):
            raise RuntimeError(
                f"Expected entry at {flattened_key} to be bytes or a tuple but received {type(obj)} instead."
            )

        # TODO: validate obj more?
        assert flattened_key is not None
        return [(flattened_key, cast(FileTypes, obj))]

    index += 1
    if is_dict(obj):
        try:
            # We are at the last entry in the path so we must remove the field
            if (len(path)) == index:
                item = obj.pop(key)
            else:
                item = obj[key]
        except KeyError:
            # Key was not present in the dictionary, this is not indicative of an error
            # as the given path may not point to a required field. We also do not want
            # to enforce required fields as the API may differ from the spec in some cases.
            return []
        if flattened_key is None:
            flattened_key = key
        else:
            flattened_key += f"[{key}]"
        return _extract_items(
            item,
            path,
            index=index,
            flattened_key=flattened_key,
        )
    elif is_list(obj):
        if key != "<array>":
            return []

        return flatten(
            [
                _extract_items(
                    item,
                    path,
                    index=index,
                    flattened_key=flattened_key + "[]" if flattened_key is not None else "[]",
                )
                for item in obj
            ]
        )

    # Something unexpected was passed, just ignore it.
    return []


# Type safe methods for narrowing types with TypeVars.
# The default narrowing for isinstance(obj, dict) is dict[unknown, unknown],
# however this cause Pyright to rightfully report errors. As we know we don't
# care about the contained types we can safely use `object` in it's place.


def is_dict(obj: object) -> TypeGuard[dict[object, object]]:
    return isinstance(obj, dict)


def is_list(obj: object) -> TypeGuard[list[object]]:
    return isinstance(obj, list)



# File generated from our OpenAPI spec by Stainless.

from .event import Event as Event
from .ledger import Ledger as Ledger
from .document import Document as Document
from .line_item import LineItem as LineItem
from .paper_item import PaperItem as PaperItem
from .transaction import Transaction as Transaction
from .counterparty import Counterparty as Counterparty
from .ledger_entry import LedgerEntry as LedgerEntry
from .payment_order import PaymentOrder as PaymentOrder
from .return_object import ReturnObject as ReturnObject
from .ledger_account import LedgerAccount as LedgerAccount
from .expected_payment import ExpectedPayment as ExpectedPayment
from .external_account import ExternalAccount as ExternalAccount
from .internal_account import InternalAccount as InternalAccount
from .event_list_params import EventListParams as EventListParams
from .ledger_list_params import LedgerListParams as LedgerListParams
from .ledger_transaction import LedgerTransaction as LedgerTransaction
from .return_list_params import ReturnListParams as ReturnListParams
from .document_list_params import DocumentListParams as DocumentListParams
from .ledger_create_params import LedgerCreateParams as LedgerCreateParams
from .ledger_update_params import LedgerUpdateParams as LedgerUpdateParams
from .return_create_params import ReturnCreateParams as ReturnCreateParams
from .line_item_list_params import LineItemListParams as LineItemListParams
from .paper_item_list_params import PaperItemListParams as PaperItemListParams
from .incoming_payment_detail import IncomingPaymentDetail as IncomingPaymentDetail
from .ledger_account_category import LedgerAccountCategory as LedgerAccountCategory
from .line_item_update_params import LineItemUpdateParams as LineItemUpdateParams
from .transaction_list_params import TransactionListParams as TransactionListParams
from .counterparty_list_params import CounterpartyListParams as CounterpartyListParams
from .ledger_entry_list_params import LedgerEntryListParams as LedgerEntryListParams
from .payment_order_list_params import PaymentOrderListParams as PaymentOrderListParams
from .transaction_update_params import (
    TransactionUpdateParams as TransactionUpdateParams,
)
from .counterparty_create_params import (
    CounterpartyCreateParams as CounterpartyCreateParams,
)
from .counterparty_update_params import (
    CounterpartyUpdateParams as CounterpartyUpdateParams,
)
from .ledger_account_list_params import (
    LedgerAccountListParams as LedgerAccountListParams,
)
from .payment_order_create_params import (
    PaymentOrderCreateParams as PaymentOrderCreateParams,
)
from .payment_order_update_params import (
    PaymentOrderUpdateParams as PaymentOrderUpdateParams,
)
from .expected_payment_list_params import (
    ExpectedPaymentListParams as ExpectedPaymentListParams,
)
from .external_account_list_params import (
    ExternalAccountListParams as ExternalAccountListParams,
)
from .internal_account_list_params import (
    InternalAccountListParams as InternalAccountListParams,
)
from .ledger_account_create_params import (
    LedgerAccountCreateParams as LedgerAccountCreateParams,
)
from .ledger_account_update_params import (
    LedgerAccountUpdateParams as LedgerAccountUpdateParams,
)
from .routing_number_lookup_request import (
    RoutingNumberLookupRequest as RoutingNumberLookupRequest,
)
from .expected_payment_create_params import (
    ExpectedPaymentCreateParams as ExpectedPaymentCreateParams,
)
from .expected_payment_update_params import (
    ExpectedPaymentUpdateParams as ExpectedPaymentUpdateParams,
)
from .external_account_create_params import (
    ExternalAccountCreateParams as ExternalAccountCreateParams,
)
from .external_account_update_params import (
    ExternalAccountUpdateParams as ExternalAccountUpdateParams,
)
from .external_account_verify_params import (
    ExternalAccountVerifyParams as ExternalAccountVerifyParams,
)
from .internal_account_create_params import (
    InternalAccountCreateParams as InternalAccountCreateParams,
)
from .internal_account_update_params import (
    InternalAccountUpdateParams as InternalAccountUpdateParams,
)
from .ledger_account_retrieve_params import (
    LedgerAccountRetrieveParams as LedgerAccountRetrieveParams,
)
from .ledger_transaction_list_params import (
    LedgerTransactionListParams as LedgerTransactionListParams,
)
from .ledger_transaction_create_params import (
    LedgerTransactionCreateParams as LedgerTransactionCreateParams,
)
from .ledger_transaction_update_params import (
    LedgerTransactionUpdateParams as LedgerTransactionUpdateParams,
)
from .payment_order_create_async_params import (
    PaymentOrderCreateAsyncParams as PaymentOrderCreateAsyncParams,
)
from .counterparty_collect_account_params import (
    CounterpartyCollectAccountParams as CounterpartyCollectAccountParams,
)
from .incoming_payment_detail_list_params import (
    IncomingPaymentDetailListParams as IncomingPaymentDetailListParams,
)
from .ledger_account_category_list_params import (
    LedgerAccountCategoryListParams as LedgerAccountCategoryListParams,
)
from .counterparty_collect_account_response import (
    CounterpartyCollectAccountResponse as CounterpartyCollectAccountResponse,
)
from .incoming_payment_detail_update_params import (
    IncomingPaymentDetailUpdateParams as IncomingPaymentDetailUpdateParams,
)
from .ledger_account_category_create_params import (
    LedgerAccountCategoryCreateParams as LedgerAccountCategoryCreateParams,
)
from .ledger_account_category_update_params import (
    LedgerAccountCategoryUpdateParams as LedgerAccountCategoryUpdateParams,
)
from .validation_validate_routing_number_params import (
    ValidationValidateRoutingNumberParams as ValidationValidateRoutingNumberParams,
)
from .external_account_complete_verification_params import (
    ExternalAccountCompleteVerificationParams as ExternalAccountCompleteVerificationParams,
)



# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from typing import TYPE_CHECKING, Union, Optional

from ..._types import NOT_GIVEN, Query, Headers, Timeout, NoneType, NotGiven
from ..._utils import extract_files
from .reversals import Reversals, AsyncReversals
from ..._resource import SyncAPIResource, AsyncAPIResource
from ...pagination import SyncPage, AsyncPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.payment_order import PaymentOrder
from ...types.payment_order_list_params import PaymentOrderListParams
from ...types.payment_order_create_params import PaymentOrderCreateParams
from ...types.payment_order_update_params import PaymentOrderUpdateParams
from ...types.payment_order_create_async_params import PaymentOrderCreateAsyncParams

if TYPE_CHECKING:
    from ..._client import ModernTreasury, AsyncModernTreasury

__all__ = ["PaymentOrders", "AsyncPaymentOrders"]


class PaymentOrders(SyncAPIResource):
    reversals: Reversals

    def __init__(self, client: ModernTreasury) -> None:
        super().__init__(client)
        self.reversals = Reversals(client)

    def create(
        self,
        body: PaymentOrderCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> PaymentOrder:
        """Create a new Payment Order"""
        # Make a copy of the input so that our internal mutations (extracting files)
        # don't incidentally mutate the user's dictionary.
        body = body.copy()
        files = extract_files(body, paths=[["documents", "<array>", "file"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            headers = {"Content-Type": "multipart/form-data", **(headers or {})}

        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            "/api/payment_orders",
            body=body,
            files=files,
            options=options,
            cast_to=PaymentOrder,
        )

    def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> PaymentOrder:
        """Get details on a single payment order"""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get(
            f"/api/payment_orders/{id}",
            options=options,
            cast_to=PaymentOrder,
        )

    def update(
        self,
        id: str,
        body: PaymentOrderUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> PaymentOrder:
        """Update a payment order"""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._patch(
            f"/api/payment_orders/{id}",
            body=body,
            options=options,
            cast_to=PaymentOrder,
        )

    def list(
        self,
        query: PaymentOrderListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> SyncPage[PaymentOrder]:
        """Get a list of all payment orders"""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/payment_orders",
            page=SyncPage[PaymentOrder],
            options=options,
            model=PaymentOrder,
        )

    def create_async(
        self,
        body: PaymentOrderCreateAsyncParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Create a new payment order asynchronously"""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return self._post(
            "/api/payment_orders/create_async",
            body=body,
            options=options,
            cast_to=NoneType,
        )


class AsyncPaymentOrders(AsyncAPIResource):
    reversals: AsyncReversals

    def __init__(self, client: AsyncModernTreasury) -> None:
        super().__init__(client)
        self.reversals = AsyncReversals(client)

    async def create(
        self,
        body: PaymentOrderCreateParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> PaymentOrder:
        """Create a new Payment Order"""
        # Make a copy of the input so that our internal mutations (extracting files)
        # don't incidentally mutate the user's dictionary.
        body = body.copy()
        files = extract_files(body, paths=[["documents", "<array>", "file"]])
        if files:
            # It should be noted that the actual Content-Type header that will be
            # sent to the server will contain a `boundary` parameter, e.g.
            # multipart/form-data; boundary=---abc--
            headers = {"Content-Type": "multipart/form-data", **(headers or {})}

        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            "/api/payment_orders",
            body=body,
            files=files,
            options=options,
            cast_to=PaymentOrder,
        )

    async def retrieve(
        self,
        id: str,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> PaymentOrder:
        """Get details on a single payment order"""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._get(
            f"/api/payment_orders/{id}",
            options=options,
            cast_to=PaymentOrder,
        )

    async def update(
        self,
        id: str,
        body: PaymentOrderUpdateParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> PaymentOrder:
        """Update a payment order"""
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._patch(
            f"/api/payment_orders/{id}",
            body=body,
            options=options,
            cast_to=PaymentOrder,
        )

    def list(
        self,
        query: PaymentOrderListParams = {},
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
    ) -> AsyncPaginator[PaymentOrder, AsyncPage[PaymentOrder]]:
        """Get a list of all payment orders"""
        options = make_request_options(headers, max_retries, timeout, query)
        return self._get_api_list(
            "/api/payment_orders",
            page=AsyncPage[PaymentOrder],
            options=options,
            model=PaymentOrder,
        )

    async def create_async(
        self,
        body: PaymentOrderCreateAsyncParams,
        *,
        headers: Union[Headers, NotGiven] = NOT_GIVEN,
        max_retries: Union[int, NotGiven] = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        query: Optional[Query] = None,
    ) -> None:
        """Create a new payment order asynchronously"""
        headers = {"Accept": "*/*", **(headers or {})}
        options = make_request_options(headers, max_retries, timeout, query)
        return await self._post(
            "/api/payment_orders/create_async",
            body=body,
            options=options,
            cast_to=NoneType,
        )
